// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "mainwindow.h"

//#include "settings_general.h"
#include "settings_general_dialog.h"
//#include "settings_general_widget_saving.h"
//#include "settings_general_widget_relayserver.h"
#include <KApplication>
#include <KAction>
#include <KLocale>
#include <KActionCollection>
#include <KStandardAction>
#include <KTipDialog>


MainWindow::MainWindow(QWidget *parent) : KXmlGuiWindow(parent)
{

  //initialise central widget
  m_streamlistwidget = new stationlistWidget(this);
  setCentralWidget(m_streamlistwidget);

  //setup actions
  KStandardAction::preferences(this, SLOT(display_global_settings_dialog()), actionCollection());

  KStandardAction::quit(kapp, SLOT(quit()), actionCollection());

  KAction* newStreamAction = new KAction(this);
  newStreamAction->setText(i18nc("KAction name (new stream)", "&New stream"));
  newStreamAction->setIcon(KIcon("document-new"));
  newStreamAction->setShortcut(QKeySequence::New);
  connect( newStreamAction, SIGNAL(triggered(bool)), m_streamlistwidget, SLOT(addNewStation()) );
  actionCollection()->addAction("newStream", newStreamAction);  // make this action avaible to the XmlGui Framework under the name
  m_streamlistwidget->contextMenu.addAction(newStreamAction);

  KAction* deleteStreamAction = new KAction(this);
  deleteStreamAction->setText(i18nc("KAction name (delete stream)", "&Delete stream"));
  deleteStreamAction->setIcon(KIcon("edit-delete"));
  deleteStreamAction->setShortcut(Qt::SHIFT+Qt::Key_Delete);
  deleteStreamAction->setDisabled(true);
  connect( deleteStreamAction, SIGNAL(triggered(bool)), m_streamlistwidget, SLOT(deleteStream()) );
  connect( m_streamlistwidget, SIGNAL(noneSelected(bool)),     deleteStreamAction, SLOT(setDisabled(bool)) );
  connect( m_streamlistwidget, SIGNAL(oneSelected(bool)),      deleteStreamAction, SLOT(setEnabled(bool))  );
  connect( m_streamlistwidget, SIGNAL(multipleSelected(bool)), deleteStreamAction, SLOT(setEnabled(bool))  );
  actionCollection()->addAction("deleteStream", deleteStreamAction);  // make this action avaible to the XmlGui Framework under the name
  m_streamlistwidget->contextMenu.addAction(deleteStreamAction);

  KAction* selectAllAction = KStandardAction::selectAll(this , SLOT(selectAll()), actionCollection() );
  m_streamlistwidget->contextMenu.addAction(selectAllAction);

  KAction* recordAction = new KAction(this);
  recordAction->setText(i18nc("KAction name (record stream)", "&Record"));
  recordAction->setIcon(KIcon("media-record"));
  recordAction->setShortcut(Qt::CTRL+Qt::Key_R);  // TODO: use a appropriate value // accept multimedia keys also
  recordAction->setDisabled(true);
  connect( recordAction, SIGNAL(triggered(bool)), m_streamlistwidget, SLOT(record()) );
  connect( m_streamlistwidget, SIGNAL(noneSelected(bool)),     recordAction, SLOT(setDisabled(bool)) );
  connect( m_streamlistwidget, SIGNAL(oneSelected(bool)),      recordAction, SLOT(setEnabled(bool))  );  // TODO: enable only when the streams are stoped...
  connect( m_streamlistwidget, SIGNAL(multipleSelected(bool)), recordAction, SLOT(setEnabled(bool))  );  // TODO: enable only when the streams are stoped...
  actionCollection()->addAction("record", recordAction);  // make this action avaible to the XmlGui Framework under the name
  m_streamlistwidget->contextMenu.addAction(recordAction);

  KAction* stopRecordAction = new KAction(this);
  stopRecordAction->setText(i18nc("KAction name (stop recording)", "&Stop"));
  stopRecordAction->setIcon(KIcon("media-playback-stop"));
  QList<QKeySequence> stopRecordAction_tempShortcutList;
  stopRecordAction_tempShortcutList.append(Qt::Key_S);
  stopRecordAction_tempShortcutList.append(Qt::CTRL+Qt::Key_S);
  stopRecordAction_tempShortcutList.append(Qt::META+Qt::Key_V);
  stopRecordAction->setShortcuts(stopRecordAction_tempShortcutList);  // TODO: use a appropriate value // accept multimedia keys also
  stopRecordAction->setDisabled(true);
  connect( stopRecordAction, SIGNAL(triggered(bool)), m_streamlistwidget , SLOT(stopRecord()) );
  connect( m_streamlistwidget, SIGNAL(noneSelected(bool)),     stopRecordAction, SLOT(setDisabled(bool)) );
  connect( m_streamlistwidget, SIGNAL(oneSelected(bool)),      stopRecordAction, SLOT(setEnabled(bool))  );  // TODO: enable only when the streams are running...
  connect( m_streamlistwidget, SIGNAL(multipleSelected(bool)), stopRecordAction, SLOT(setEnabled(bool))  );  // TODO: enable only when the streams are running...
  actionCollection()->addAction("stopRecord", stopRecordAction);  // make this action avaible to the XmlGui Framework under the name
  m_streamlistwidget->contextMenu.addAction(stopRecordAction);

  KAction* streamSettingsAction = new KAction(this);
  streamSettingsAction->setText(i18nc("KAction name (per stream settings)", "&Properties"));
  //stopRecordAction->setIcon(KIcon("media-playback-stop")); // TODO: use a appropriate value
  streamSettingsAction->setShortcut(Qt::ALT+Qt::Key_Return);  // TODO: use a appropriate value // accept multimedia keys also
  streamSettingsAction->setDisabled(true);
  connect( streamSettingsAction, SIGNAL(triggered(bool)), m_streamlistwidget , SLOT(displayStreamSettings()) );
  connect( m_streamlistwidget, SIGNAL(noneSelected(bool)),     streamSettingsAction, SLOT(setDisabled(bool)) );
  connect( m_streamlistwidget, SIGNAL(oneSelected(bool)),      streamSettingsAction, SLOT(setEnabled(bool))  );
  connect( m_streamlistwidget, SIGNAL(multipleSelected(bool)), streamSettingsAction, SLOT(setDisabled(bool))  );  // TODO: maybe make this possible later...
  actionCollection()->addAction("streamSettings", streamSettingsAction);  // make this action avaible to the XmlGui Framework under the name
  m_streamlistwidget->contextMenu.addAction(streamSettingsAction);

  KStandardAction::tipOfDay(this , SLOT(displayTipOfDay()), actionCollection() );

  //setup GUI
  setupGUI();
  KTipDialog::showTip(this, "kradioripper/tips");
}

MainWindow::~MainWindow()
{
}

void MainWindow::display_global_settings_dialog()
{
  // if the dialog object yet exists, we can display it directly and return.
  if (KConfigDialog::showDialog("settings_general_dialog"))
    return;

  // else we have to construct and configure it befor displaying it and return
  settings_general_dialog* dialog = new settings_general_dialog(this, "settings_general_dialog");
  dialog->show();
}

void MainWindow::selectAll()
{
  m_streamlistwidget->clearSelection();
  m_streamlistwidget->selectAll();
}

void MainWindow::displayTipOfDay()
{
  KTipDialog::showTip(this, "kradioripper/tips", true);
}
