// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#ifndef GET_STREAM_INFO_H
#define GET_STREAM_INFO_H

#include "ripping.h"

/** \brief This class detects meta data from streams.
*
* This class calls \e %streamripper, lets it connect to the stream, recognizes
* #streamName, #serverName, #bitrate and #metaInterval and disconnects immediately after that.
*
* Just set the property #serverUri and startStreamripper(). The recognized data is avaible
* as usual through the properties. */
class get_stream_info : public ripping
{



     Q_OBJECT



     /** This property holds the URI to which \e %streamripper will connect.
     *
     * <b> Initialization value: </b> <EM> an empty string </EM>
     *
     * \li \e read: \code virtual QString serverUri() const \endcode This (public) function
     *              implements <tt> ripping::serverUri() </tt> (which was originally protected).
     *              At the same time it is used as read function for this property.
     * \li \e write: \code virtual void setServerUri(const QString& serverUri) \endcode
     *
     * \sa \code qint64 internal_serverUri \endcode */ 
     Q_PROPERTY(QString serverUri READ serverUri WRITE setServerUri)



public:
     //contructor(s) and destructor(s)
     /** Constructor of the class.
     * @param parent Sets the parent of this object. */
     get_stream_info(const QPointer<QObject> parent = 0);
     /** The destructor. */
     virtual ~get_stream_info();
     /** See property #serverUri. */
     virtual QString serverUri() const;  // this pure virtual function from "ripping" must be implemented!  // was originally PROTECTED
     /** See property #serverUri. */
     virtual void setServerUri(const QString& serverUri);



protected:
     /** Returns the parameters which <tt>virtual QStringList ripping::parameterList() const</tt> would return and
     *   addititionaly a parameter that stops \e %streamripper directly after the first data is send and
     *   a parameter that prevents \e %streamripper from writing any data. */
     virtual QStringList parameterList() const;



private:
     /** Used internally to store the property #serverUri. */
     QString internal_serverUri;



}; 

#endif
