// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "ripping.h"
#include "settings_general.h"
#include "m_header.h"

#include <KGlobal>
#include <KLocale>
#include <QFileInfo>

ripping::ripping(const QPointer<QObject> parent) : streamripper_base(parent)
{
  internal_splitBehavior = QString::SkipEmptyParts;  // makes shure that interpretate_console_output() doesn't get empty lines (which wouldn't make any sense, but only consume CPU time)

  /* The internal variables who hold the properties get initialized. (The variables of type "QString" don't need
  *  any initialatiation because they have yet an empty string as default.) We write directly to the variables
  *  and don't use the setPROPERTY() methods because this way the initialatation has no effect when setPROPERTY() is
  *  reimplemented in inherited classes. (That's important because the inherited class "radioStation" uses config files
  *  to hold the data of the properties instead of the variables - and when the program starts, "bitrate" shouldn't
  *  be reseted to -2 when the config file holds yet the correct value known from the last call.) */
  internal_dataSize = -2;
  internal_relayPort = -2;
  internal_bitrate = -2;
  internal_metaInterval = -2;

  QObject::connect( &m_process,
                    SIGNAL( stateChanged(QProcess::ProcessState) ),
                    this,
                    SLOT( streamripperStateChange(QProcess::ProcessState) )
                  );
  QObject::connect( &m_process,
                    SIGNAL( error(QProcess::ProcessError ) ),
                    this,
                    SLOT( errorOccured(QProcess::ProcessError ) )
                  );
}

ripping::~ripping()
{
}

void ripping::interpretate_console_output(QStringList& stringList)
{
  // variables
  QString my_line;
  // recognized data
  bool statusSet=FALSE;  // this means that setStatus() AND setSong() AND setDataSize() where called!
  bool relayPortSet=FALSE;
  bool streamNameSet=FALSE;
  bool serverNameSet=FALSE;
  bool bitrateSet=FALSE;
  bool metaIntervalSet=FALSE;
  bool errorSet=FALSE;
  // help variables
  QString help_string;
  bool okay;
  qint64 helper_qint64;
  

  // code
  // recognize data
  while ( !stringList.isEmpty() )
  {
    my_line = stringList.takeLast();  // gives back the last list entry and deletes it


    // We can't to "my_line=my_line.toLower();" befor (what would be practical to not do this in each "if" condition)
    // because we need the original string with upper and lower cases when we want to recognize a song title!
    if ( my_line.toLower().startsWith("[ripping...    ] ") )
    {
      if (!statusSet)
      { 
        setStatus( i18nc("process status","Ripping...") );
        my_line = my_line.remove(0, 17); // remove "[ripping...    ] " from the begin of the string
        helper_interpretate_metainfo_and_datasize(my_line);
        statusSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("[buffering - | ] ") )
    {
      if (!statusSet)
      {
        setStatus( i18nc("process status","Buffering...") );
        my_line = my_line.remove(0, 17); // remove "[buffering - | ] " from the begin of the string
        helper_interpretate_metainfo_and_datasize(my_line);
        statusSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("[skipping...   ] ") )
    {
      if (!statusSet)
      {
        setStatus( i18nc("process status","Skipping...") );
        my_line = my_line.remove(0, 17); // remove "[skipping...   ] " from the begin of the string
        helper_interpretate_metainfo_and_datasize(my_line);
        statusSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("connecting") OR my_line.toLower().startsWith("[getting track name") )
    {
      if (!statusSet)
      {
        setStatus( i18nc("process status","Connecting...") );
        setSong("");
        setDataSize(-2);
        statusSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("shutting down") OR my_line.toLower().startsWith("bye..") )
    {
      if (!statusSet)
      {
        setStatus( i18nc("process status","Saving files...") );
        setSong("");
        setDataSize(-2);
        statusSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("relay port: ") )
    {
      if (!relayPortSet)
      {
        helper_qint64 = my_line.right(my_line.size()-12).toLongLong(&okay);
        if (NOT okay) setRelayPort(-1);
        else setRelayPort(helper_qint64);
        relayPortSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("stream: ") )
    {
      if (!streamNameSet)
      {
        setStreamName( my_line.right(my_line.size()-8) );
        streamNameSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("server name: ") )
    {
      if (!serverNameSet)
      {
        setServerName( my_line.right(my_line.size()-13) );
        serverNameSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("bitrate: ") )
    {
      if (!bitrateSet)
      {
        helper_qint64 = my_line.right(my_line.size()-9).toLongLong(&okay);
        if (NOT okay) setBitrate(-1);
        else setBitrate(helper_qint64);
        bitrateSet = TRUE;
      };
    }
    else if ( my_line.toLower().startsWith("meta interval: ") )
    {
      if (!metaIntervalSet)
      {
        helper_qint64 = my_line.right(my_line.size()-15).toLongLong(&okay);
        if (NOT okay) setMetaInterval(-1);
        else setMetaInterval(helper_qint64);
        metaIntervalSet = TRUE;
      }
    }
    else if ( my_line.toLower().startsWith("error -") )
    {
      if (!errorSet)
      {
        setError( helper_formatedErrorMessage(my_line) );
        errorSet = TRUE;
      };
    }
    else
    {
      kDebug() << "Konnte folgenden String nicht erkennen:" << my_line << "size:" << my_line.size() << endl;
    }
  };
}

void ripping::helper_interpretate_metainfo_and_datasize(QString my_line) // TODO Option: nur metainfo, nicht dataSize erkennen (geht sonst schief!)
{
  // help variables
  QString help_string;
  bool okay;
  qint64 dataSize;

  // code
  help_string = my_line.section('[', -1);  // take all characters behind the last "[" in the string and put it to help_string  // TODO works only if there _is_ also a data size in the string (not the case when "buffering..."!)
  my_line.truncate( my_line.size() - help_string.size() - 2 ); // removes the help_string from the end and also " [" befor the help_string -> so there stays only the song (Okay, it's not really the song. It's the metadata.) itself.

  // metainfo
  if (my_line EQUAL " - ")
    setSong("");
  else
    setSong(my_line);

  // data size
  help_string=help_string.remove(']').simplified().toUpper(); // remove ']' and whitespace and convert tu uppercase letters
  if ( help_string.endsWith("KB") OR help_string.endsWith("K") )
  {
    if ( help_string.endsWith("KB") )
      help_string.truncate( help_string.size() - 2 );
    else
      help_string.truncate( help_string.size() - 1 );
    dataSize = help_string.toLongLong(&okay) * 1024;
  }
  else 
    if ( help_string.endsWith("MB") OR help_string.endsWith("M") )
    {
      if ( help_string.endsWith("MB") )
        help_string.truncate( help_string.size() - 2 );
      else
        help_string.truncate( help_string.size() - 1 );
      dataSize = help_string.toDouble(&okay) * 1024 * 1024;
    }
    else
      if ( help_string.endsWith("B") )  // ends with "B", but not "MB" or "KB" (see previous contitions)
      {
        help_string.truncate( help_string.size() - 1 );
        dataSize = help_string.toLongLong(&okay);
      }
      else
        okay = false;
  if (NOT okay) setDataSize(-1);
  else setDataSize(dataSize);
}

inline QString ripping::helper_formatedErrorMessage(const QString& streamripperOutputLine)
{
  return streamripperOutputLine; // TODO implement this _really_
// "[getting track name... ]"
//error -6 [SR_ERROR_CANT_RESOLVE_HOSTNAME]
}

QString ripping::streamName() const
{
  return internal_streamName;
}

void ripping::setStreamName(const QString& streamName)
{
  if (internal_streamName != streamName)
  {
    internal_streamName=streamName;
    emit streamNameChanged(index(), streamName);
  };
}

QString ripping::serverName() const
{
  return internal_serverName;
}

void ripping::setServerName(const QString& serverName)
{
  if (internal_serverName != serverName)
  {
    internal_serverName=serverName;
    emit serverNameChanged(index(), serverName);
  };
}

QString ripping::status() const
{
  return internal_status;
}

void ripping::setStatus(const QString& status)
{
  if (internal_status != status)
  {
    internal_status=status;
    emit statusChanged(index(), status);
  };
}

QString ripping::error() const
{
  return internal_error;
}

void ripping::setError(const QString& error)
{
  if (internal_error != error)
  {
    internal_error=error;
    emit errorChanged(index(), error);
  };
}

QString ripping::song() const
{
  return internal_song;
}

void ripping::setSong(const QString& song)
{
  if (internal_song != song)
  {
    internal_song=song;
    emit songChanged(index(), song);
  };
}

qint64 ripping::dataSize() const
{
  return internal_dataSize;
}

void ripping::setDataSize(qint64 dataSize)
{
  if (internal_dataSize != dataSize)
  {
    internal_dataSize=dataSize;
    emit dataSizeChanged(index(), dataSize);
  };
}

qint64 ripping::relayPort() const
{
  return internal_relayPort;
}

void ripping::setRelayPort(qint64 relayPort)
{
  if (internal_relayPort != relayPort)
  {
    internal_relayPort=relayPort;
    emit relayPortChanged(index(), relayPort);
  };
}

qint64 ripping::bitrate() const
{
  return internal_bitrate;
}

QString ripping::format_bitrate(long bitrate)
{
  // variables
  QString temp;

  // code
  if (bitrate >= 1)
    temp = i18ncp("This makes a nicly formated string for the bitrate of a stream - %1 is an integer.",
                  "%1 kbit/s",
                  "%1 kbit/s",
                  bitrate); // TODO why is the number format always english / why is the locale alsways english?
  else
    if (bitrate >= -1)
      temp = i18nc("This makes a nicly formated string for the bitrate of a stream.", "Unable to recognize bitrate.");
    //else (bitrate <= -2) means: no value set. So an empty string is returned.
  return temp;
// TODO why doesn't works it to use "," instead of "." as Dezimaltrenner?
}

void ripping::setBitrate(qint64 bitrate)
{
  if (internal_bitrate != bitrate)
  {
    internal_bitrate=bitrate;
    emit bitrateChanged(index(), bitrate);
  };
}

qint64 ripping::metaInterval() const
{
  return internal_metaInterval;
}

QString ripping::format_metaInterval(long metaInterval)
{
  // variables
  QString temp;

  // code
  if (metaInterval > 0)     // metaInterval is 1 or bigger
    temp = i18ncp("just a locally formated number: the formated version of the meta interval!", "%1", "%1", metaInterval ); // TODO what unit is this value of? Insert also the unit!
  else
    if (metaInterval >= -1) // metaInterval is 0 or -1
      temp = i18nc("This makes a nicly formated string for the metaInterval of a stream.", "Unable to read meta data.");
  // else: when metaInterval is -2 (or smaller), then no info is avaible.
  return temp;
}

void ripping::setMetaInterval(qint64 metaInterval)
{
  if (internal_metaInterval != metaInterval)
  {
    internal_metaInterval=metaInterval;
    emit metaIntervalChanged(index(), metaInterval);
  };
}

void ripping::errorOccured( QProcess::ProcessError error)
{
  // variables
  QFileInfo m_file_info(settings_general::streamripperCommand());

  // code
  switch (error)
  {
    case QProcess::FailedToStart:
      if (m_file_info.exists())
        setError(i18nc("@info:status streamripper error", "Insufficient permissions to invoke streamripper."));
      else
        setError(i18nc("@info:status streamripper error", "Streamripper binary not found."));
      break;
    case QProcess::Crashed:
      setError(i18nc("@info:status streamripper error", "Streamripper crashed."));
      break;
    case QProcess::Timedout:
      setError(i18nc("@info:status streamripper error", "Streamripper doesn't react."));
      break;
    default:  //ReadError, WriteError, UnknownError
      setError(i18nc("@info:status streamripper error", "Error accessing Streamripper."));
      break;
  };
}

void ripping::streamripperStateChange(QProcess::ProcessState newState)
{
  if ( newState == QProcess::NotRunning )
  {
    setStatus("");
    setSong("");
    /* Here, we can't determinate
    *  - if the programm has terminated normally (good) or
    *  - if it has crashed or
    *  - if it couldn't even be started, because the binary wasn't found or wasn't executable.
    *
    *  In the last 2 cases, we should display an error message, depending on _which_ error occured.
    *  But we can't use m_process.error() to determinate the error type, because QProcess emits the
    *  signal stateChanged() (to which this slot is connected) _befor_ it actualizes the property error().
    *  Because of this, the error message is set by the slot errorOccured(), who is connected to the
    *  signal error(). */
    setDataSize(-2);
    setRelayPort(-2);
    emit not_running();
  };
  // else m_process is starting or running, and therefor this class "ripping" takes yet care of messages...
}

QStringList ripping::parameterList() const
{
  // variables
  QStringList parameters;

  //code
  parameters.append(serverUri());

  parameters.append("-t"); // don't override files in the incomplete dir
                           // but make instead a save copy of the old file
                           // by appending an index. I'm not shure if this
                           // is absolutly needed, but at least it's not bad.

  if (   NOT ( settings_general::proxyServer().isEmpty() )   ) 
  {  // use proxy server TODO use the KDE-wide proxy setting instead of this one
    parameters.append("-p");
    parameters.append(settings_general::proxyServer());
  }

  if (settings_general::createRelayServer()) // create relay server
  {
    // create a relay server at (or up to) port X!
    parameters.append("-r");
    parameters.append(QString::number(settings_general::preferedPortForRelayServer()));

    // the number of connections is limited to which number?
    parameters.append("-R");
    if (settings_general::limitConnections())
      parameters.append(QString::number(settings_general::limitConnectionsToX()));
    else
      parameters.append("0");
  };

  parameters.append("-c");  // don't auto-reconnect  TODO: implement this on my own way!

  return parameters;
}

void ripping::startStreamripper()
{
  if (m_process.state() == QProcess::NotRunning)  // TODO else if the process is yet shutting down or has yet an error -> einreihen für späteren automatischen Neustart!
  {
    setError(QString());
    setStatus(i18nc("process status","Starting..."));
    //TODO: exists the working directory? when not -> create it. Is it writable? When not -> error message.
    m_process.setWorkingDirectory(settings_general::saveDirectory().toLocalFile());
    streamripper_base::startStreamripper();
  };
}

void ripping::shutDown()
{
  m_process.terminate();
  // TODO eventuelle Einreihung für Neustart wieder löschen! (wenn jemand startet, während gerade runtergefahren wird)
  // TODO reimplementiere auch kill()
}
