// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "settings_stream_widget_connection.h"
#include "ripping.h"

#define AND  && 
#define OR  || 
#define NOT  ! 
 
settings_stream_widget_connection::settings_stream_widget_connection(QWidget *parent) : QStackedWidget(parent)
{
  setupUi(this);
  connect( kcfg_serverUri, SIGNAL(textEdited(const QString&)), this, SLOT(serverUriChanged()) );  // TODO: Warum textEdited und nicht textChanged? Denn wenn ich auf "Default" drücke, wird nix neues in Gang gesetzt. Aber mit textChanged würde ich eine Endlosschleife konstruieren, weil ich ja auch mal selbst kcfg_serverUri setzen muss! Wie löse ich das?
  connect( &m_process, SIGNAL(streamNameChanged(qlonglong, QString)),  this, SLOT(setStreamName(qlonglong, QString))  );
  connect( &m_process, SIGNAL(serverNameChanged(qlonglong, QString)),  this, SLOT(setServerName(qlonglong, QString))  );
  connect( &m_process, SIGNAL(bitrateChanged(qlonglong, qint64)),      this, SLOT(setBitrate(qlonglong, qint64))      );
  connect( &m_process, SIGNAL(metaIntervalChanged(qlonglong, qint64)), this, SLOT(setMetaInterval(qlonglong, qint64)) );
  connect( &m_process, SIGNAL(not_running()),                          this, SLOT(recognizationFinished())            );
}

// TODO nach 15 sec oder welcher Zeit auch immer sollte die Erkennung abgebrochen werden... Timeout!

settings_stream_widget_connection::~settings_stream_widget_connection()
{
}

void settings_stream_widget_connection::serverUriChanged()
{
  // preparing the m_process (which is maybe running) without that there are changes in the GUI
  disconnect( &m_process, 
           SIGNAL(not_running()),
           this,
           SLOT(recognizationFinished())
         );
  m_process.kill();     
  m_process.setServerUri(kcfg_serverUri->text());
  m_process.waitForFinished();   // to make shure the kill() is yet applied! Must be here to make shure that no more SIGNALs like changed stream name and so on will occur and change our UI.
  connect( &m_process,         
           SIGNAL(not_running()),
           this,
           SLOT(recognizationFinished())
         );

  // delete the values in the config system
  kcfg_info_serverName->setText("");
  kcfg_info_streamName->setText("");
  kcfg_info_bitrate->setValue(-2);
  kcfg_info_metaInterval->setValue(-2);

  // update the GUI
  info_serverName->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  info_streamName->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  info_bitrate->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  info_metaInterval->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  info_serverName->setEnabled(false);
  info_streamName->setEnabled(false);
  info_bitrate->setEnabled(false);
  info_metaInterval->setEnabled(false);

  // start the recognization
  m_process.startStreamripper();
}

// TODO what does this class, if the streamripper-binary isn't found?

void settings_stream_widget_connection::load_info_from_kcfg()
{
  // preparing the m_process (which is maybe running) without that there are changes in the GUI
  disconnect( &m_process,           
           SIGNAL(not_running()),
           this,
           SLOT(recognizationFinished())
         );
  m_process.kill();    
  m_process.waitForFinished();   // to make shure the kill() is yet applied! Must be here to make shure that no more SIGNALs like changed stream name and so on will occur and change our UI.
  connect( &m_process,
           SIGNAL(not_running()),
           this,
           SLOT(recognizationFinished())
         );

  if (kcfg_info_serverName->text().isEmpty())  // not set
  {
    info_serverName->setEnabled(false);
    info_serverName->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  }
  else
    setServerName(0, kcfg_info_serverName->text());

  if (kcfg_info_streamName->text().isEmpty())  // not set
  {
    info_streamName->setEnabled(false);
    info_streamName->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  }
  else
    setStreamName(0, kcfg_info_streamName->text());

  if (kcfg_info_bitrate->value() < 0)  // not set
  {
    info_bitrate->setEnabled(false);
    info_bitrate->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  }
  else
    setBitrate(0, kcfg_info_bitrate->value());

  if (kcfg_info_metaInterval->value() < -1)  // not set
  {
    info_metaInterval->setEnabled(false);
    info_metaInterval->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Recognizing..."));
  }
  else
    setMetaInterval(0, kcfg_info_metaInterval->value());

  // start the recognization. We do this - also when all metadata is present - to actualize it when possible.
  m_process.setServerUri(kcfg_serverUri->text());
  m_process.startStreamripper();
}

void settings_stream_widget_connection::setStreamName(qlonglong index, QString streamName)
{ 
  kcfg_info_streamName->setText(streamName);
  info_streamName->setText(streamName);
  if (streamName.isEmpty())
    info_streamName->setEnabled(false);
  else
    info_streamName->setEnabled(true);
}

void settings_stream_widget_connection::setServerName(qlonglong index, QString serverName)
{ 
  kcfg_info_serverName->setText(serverName);
  info_serverName->setText(serverName);
  info_serverName->setEnabled(true);
  if (serverName.isEmpty())
    info_serverName->setEnabled(false);
  else
    info_serverName->setEnabled(true);
}

void settings_stream_widget_connection::setBitrate(qlonglong index, qint64 bitrate)
{
kDebug() << "!!!!!!!!!!!!!!!!!!!!!!will set this bitrate:" << bitrate << ripping::format_bitrate(bitrate) << endl;
  kcfg_info_bitrate->setValue(bitrate);
  info_bitrate->setText(ripping::format_bitrate(bitrate));
  if (bitrate<1)
    info_bitrate->setEnabled(false);
  else
    info_bitrate->setEnabled(true);
}

void settings_stream_widget_connection::setMetaInterval(qlonglong index, qint64 metaInterval)
{ 
  kcfg_info_metaInterval->setValue(metaInterval);
  info_metaInterval->setText(ripping::format_metaInterval(metaInterval));
  info_metaInterval->setEnabled(true);
  if (metaInterval<1)
    info_metaInterval->setEnabled(false);
  else
    info_metaInterval->setEnabled(true);
}

void settings_stream_widget_connection::recognizationFinished()
{
  if (NOT info_serverName->isEnabled())
    info_serverName->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Connection failed."));
  if (NOT info_streamName->isEnabled())
    info_streamName->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Connection failed."));
  if (kcfg_info_bitrate->value() <= -2)
    info_bitrate->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Connection failed."));
  if (kcfg_info_metaInterval->value() <= -2)
    info_metaInterval->setText(i18nc("This is displayed in the stream settings dialog for the stream info after entering a new URI","Connection failed."));
}

