// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "streamripper_base.h"
#include "settings_general.h"

streamripper_base::streamripper_base(const QPointer<QObject> parent) : console_reader(parent)
{
  resetIndex();  // uses the virtual function resetIndex() to determinate the default value of the property index.
}

streamripper_base::streamripper_base(const QPointer<QObject> parent, const qlonglong &index) : console_reader(parent)
{
  setIndex(index);  // better than setting internal_index directly. And we don't have to worry about signals
                    // that are emited because this is the constructur - so there won't be any slot connected
                    // to them.
}

streamripper_base::~streamripper_base()
{
}

qlonglong streamripper_base::index() const
{
  return internal_index;
}

void streamripper_base::setIndex(const qlonglong& index)
{
  if (internal_index != index)
  {
    internal_index=index;
    emit indexChanged(index);
  };
}

void streamripper_base::resetIndex()
{
  setIndex(0);
}

void streamripper_base::startStreamripper()
{
  QStringList command_line;
  command_line.append(settings_general::streamripperCommand());
  command_line += parameterList();
  m_process.setProgram(command_line);       //prepare KProcess
  m_process.clearEnvironment();  // (almost) all enviroment variables are cleard. We don't want that strange, undefined things will happen with streamripper...
  m_process.start();             //call streamripper
}

void streamripper_base::kill()
{
  m_process.kill();              // sends the SIGKILL signal
}

void streamripper_base::waitForFinished()
{
  m_process.waitForFinished();   // we have to wait until the process really is finished - this takes only some milliseconds...
}