// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "console_reader.h"

console_reader::console_reader(const QPointer<QObject> parent) : QObject(parent)
{
  internal_splitBehavior = QString::KeepEmptyParts;
  //set up m_process
  m_process.setOutputChannelMode(KProcess::MergedChannels); //merge stderr into stdout 
  QObject::connect( &m_process,
                    SIGNAL( readyRead() ),
                    this,
                    SLOT( read_console_output() )
                  );
}

console_reader::~console_reader()
{
  m_process.kill();              // sends the SIGKILL signal
  m_process.waitForFinished();   // we have to wait until the process really is killed - this takes only some milliseconds...
}

QString console_reader::QByteArray_to_QString(QByteArray& m_byteArray)
{
  return QString::fromLocal8Bit(m_byteArray);
}

void console_reader::read_console_output()
{
  // variables
  QByteArray temp_bytearray;
  bool newLineAtTheEnd;
  QStringList m_output_lines;

  // code

  /* Append the process output data to our internal buffer: m_uncomplete_line.
  * Because readAllStandardOutput() delivers a Bytearray with the data
  * who's actually avaible, that's not always exactly 1 line, but maybe
  * a half line or various (and a half) lines - because of that,
  * we need this buffer. */
  temp_bytearray = m_process.readAllStandardOutput();
  m_uncomplete_line.append(QByteArray_to_QString(temp_bytearray));  // attention: QByteArray_to_QString could modify the bytearray. Don't use the bytearray any more!
  /* Use a unified way for "new line": First replace "\x0D\x0A" by "\x0A",
  *  than replace also all single "\x0D" by "\x0A". Doing it in this order
  *  garanties that we have later exactly as many line breaks as befor!
  *
  *  I'm not shure if C++ doesn't interpretate \n on Windows platform sometimes
  *  as (13, 10) instead of (10), so using hex numbers I make shure that
  *  that isn't relevant. */
  m_uncomplete_line.replace("\x0D\x0A", "\x0A");
  m_uncomplete_line.replace("\x0D", "\x0A");

  // now process the data in our internal buffer
  newLineAtTheEnd = m_uncomplete_line.endsWith(10);
  m_output_lines=m_uncomplete_line.split(10, internal_splitBehavior);
  if ( m_output_lines.isEmpty() )
    m_uncomplete_line=""; // the internal buffer is made empty (possibly there where \n characters...)
  else
  {
    if ( !newLineAtTheEnd )  // in this case, the last line in the buffer isn't yet completly written by the command line program. So this last line stays in the buffer waiting for being completed the next time when the process writes data. (The other - complete - lines, saved in m_output_lines, will be processed.)
    {
      m_uncomplete_line=m_output_lines.takeLast(); // removes the last item from the list and returns it -> here saving it to the buffer
      if ( ! m_output_lines.isEmpty() )
        interpretate_console_output(m_output_lines);  // process the data // attention: interpretate_console_output could modify m_output_lines. Don't use m_output_lines any more!
    }
    else
    {
      m_uncomplete_line="";  // in this case, the last line is completly written. So the buffer is made empty, because all the data will be processed now.
      interpretate_console_output(m_output_lines);  // process the data // attention: interpretate_console_output could modify my_output_lines. Don't use my_output_lines any more!
    };
  };
}
