// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "stationlistmodel.h"
#include "settings_general.h"
#include <QFile>
#include <klocalizedstring.h>
#include <klocale.h>

#include "m_header.h"


stationlistModel::stationlistModel(const QPointer<QObject> parent, const QPointer<QWidget> mainWidget) : QAbstractTableModel(parent)
{
//TODO: check if yet one object exists. When yes, don't create another object. Es darf nur ein einziges Objekt dieser Klasse gleichzeitig geben!

  // variables
  QStringList m_stringList;
  int i;  // using int because this is the return type of QStringList::size()

  // general setup
  m_mainWidget = mainWidget;
  internal_playing = -1;
  internal_actuallyPlaybackedPort = -1;

  // setup music playback
  m_mediaObject = new Phonon::MediaObject(this);
  m_audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
  m_path = Phonon::createPath(m_mediaObject, m_audioOutput);
  connect(m_mediaObject,
          SIGNAL(stateChanged ( Phonon::State , Phonon::State )),
          this,
          SLOT(process_new_mediaObjectState ( Phonon::State ))
         );  // see actualizeAudioPlayer for details

  // setup data model
  m_stringList = settings_general::streamConfigFiles();
//kDebug() << "construktor m_stringList" << m_stringList << endl;
  for (i = 0; i < m_stringList.size(); ++i)
  {
    m_stationlist.append(new radioStation(this, m_mainWidget, m_stringList.at(i), i));
    connectSignalsAndSlots(m_stationlist.last());
  };
  writeStationListToGeneralSettings();  // This is because when radioStation can't open the file, it creates a new one.
                                        // Probaby this won't happen, but we have to make shure that the list in the general config file is actual.
}

stationlistModel::~stationlistModel()
{
}

void stationlistModel::connectSignalsAndSlots(QPointer<radioStation> m_stream)
{
    connect(m_stream, SIGNAL(streamNameChanged(qlonglong, QString)),
            this, SLOT(reloadStreamName(qlonglong)));
    connect(m_stream, SIGNAL(statusChanged(qlonglong, QString)),
            this, SLOT(reloadStatus(qlonglong)));
    connect(m_stream, SIGNAL(errorChanged(qlonglong, QString)),
            this, SLOT(reloadStatus(qlonglong)));
    connect(m_stream, SIGNAL(songChanged(qlonglong, QString)),
            this, SLOT(reloadSong(qlonglong)));
    connect(m_stream, SIGNAL(dataSizeChanged(qlonglong, qint64)),
            this, SLOT(reloadDataSize(qlonglong)));
    connect(m_stream, SIGNAL(bitrateChanged(qlonglong, qint64)),
            this, SLOT(reloadBitrate(qlonglong)));
    connect(m_stream, SIGNAL(metaIntervalChanged(qlonglong, qint64)),
            this, SLOT(reloadMetaInterval(qlonglong)));
    connect(m_stream, SIGNAL(serverNameChanged(qlonglong, QString)),
            this, SLOT(reloadServerName(qlonglong)));
    connect(m_stream, SIGNAL(relayPortChanged(qlonglong, qint64)),
            this, SLOT(reloadRelayPort(qlonglong)));
}

int stationlistModel::rowCount (const QModelIndex & parent) const
{
  return m_stationlist.size();
}

int stationlistModel::columnCount (const QModelIndex & parent) const
{
  return 8;
}

QVariant stationlistModel::columnInfo(const int & column, const columnInfoType & type, const qint64 & value) const
{
  switch (column)
  {



    case 0:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return i18nc("", "stream name");
        case columnHeaderToolTip:
          return i18nc("", "");
        case columnHeaderWhatsThis:
          return i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_streamName();
        case setColumnWidth:
          settings_general::setColumnWidth_streamName(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_streamName();
        case setColumnVisibility:
          settings_general::setColumnVisibility_streamName(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          return m_stationlist.at(value)->streamName();
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 1:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return i18nc("", "status");
        case columnHeaderToolTip:
          return i18nc("", "");
        case columnHeaderWhatsThis:
          return i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_statusAndError();
        case setColumnWidth:
          settings_general::setColumnWidth_statusAndError(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_statusAndError();
        case setColumnVisibility:
          settings_general::setColumnVisibility_statusAndError(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          if ( m_stationlist.at(value)->status().isEmpty() )
            return m_stationlist.at(value)->error();
          else if ( m_stationlist.at(value)->error().isEmpty() )
            return m_stationlist.at(value)->status();
          else
            return QString(i18nc("This produces the status message from the actual status (%1) and the error message (%3). Both of them are guarenteed to be not empty. %2 is replayed by an en-dash (Unicode U+2013). Use it or replace it by something what looks nicer in your language.", "%1 %2 %3")).arg(m_stationlist.at(value)->status()).arg(m_stationlist.at(value)->error()).arg(QChar(0x2013));
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 2:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return  i18nc("", "song");
        case columnHeaderToolTip:
          return  i18nc("", "");
        case columnHeaderWhatsThis:
          return  i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_song();
        case setColumnWidth:
          settings_general::setColumnWidth_song(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_song();
        case setColumnVisibility:
          settings_general::setColumnVisibility_song(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          return m_stationlist.at(value)->song();
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 3:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return  i18nc("", "song size");
        case columnHeaderToolTip:
          return  i18nc("", "");
        case columnHeaderWhatsThis:
          return  i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_dataSize();
        case setColumnWidth:
          settings_general::setColumnWidth_dataSize(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_dataSize();
        case setColumnVisibility:
          settings_general::setColumnVisibility_dataSize(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          if ( m_stationlist.at(value)->dataSize() <= 0 )
            return QVariant();
          else
            return i18nc("",
                         "%1 MB",
                         KGlobal::locale()->formatNumber(
                           double(m_stationlist.at(value)->dataSize()) / (1024*1024),
                           2   )
                        );
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 4:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return  i18nc("", "bit rate");
        case columnHeaderToolTip:
          return  i18nc("", "");
        case columnHeaderWhatsThis:
          return  i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_bitrate();
        case setColumnWidth:
          settings_general::setColumnWidth_bitrate(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_bitrate();
        case setColumnVisibility:
          settings_general::setColumnVisibility_bitrate(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          if ( m_stationlist.at(value)->bitrate() <= 0 )
            return QVariant();
          else
            return m_stationlist.at(value)->bitrate();
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 5:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return  i18nc("", "meta interval");
        case columnHeaderToolTip:
          return  i18nc("", "interval of meta data");
        case columnHeaderWhatsThis:
          return  i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_metaInterval();
        case setColumnWidth:
          settings_general::setColumnWidth_metaInterval(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_metaInterval();
        case setColumnVisibility:
          settings_general::setColumnVisibility_metaInterval(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          if ( m_stationlist.at(value)->metaInterval() <= 0 )
            return QVariant();
          else
            return m_stationlist.at(value)->metaInterval();
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 6:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return  i18nc("", "server");
        case columnHeaderToolTip:
          return  i18nc("", "name of the server");
        case columnHeaderWhatsThis:
          return  i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_serverName();
        case setColumnWidth:
          settings_general::setColumnWidth_serverName(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_serverName();
        case setColumnVisibility:
          settings_general::setColumnVisibility_serverName(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          return m_stationlist.at(value)->serverName();
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



    case 7:
    {
      switch (type)
      {
        case columnHeaderTitle:
          return  i18nc("", "relay port");
        case columnHeaderToolTip:
          return  i18nc("", "port of the relay server");
        case columnHeaderWhatsThis:
          return  i18nc("", "");
        case columnWidth:
          return settings_general::columnWidth_relayPort();
        case setColumnWidth:
          settings_general::setColumnWidth_relayPort(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnVisibility:
          return settings_general::columnVisibility_relayPort();
        case setColumnVisibility:
          settings_general::setColumnVisibility_relayPort(value);
          settings_general::self()->writeConfig();
          return QVariant();
        case columnData:
          if ( m_stationlist.at(value)->relayPort() <= 0 )
            return QVariant();
          else
            return m_stationlist.at(value)->relayPort();
        case columnDataToolTip:
          return i18nc("", "");
        case columnDataWhatsThis:
          return i18nc("", "");
      };
      break;
    }



  };
  return QVariant();  // just to be absolutly shure that "return" is called ;-)
}

void stationlistModel::reloadStreamName(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 0), index(stationIndex, 0) );
}

void stationlistModel::reloadStatus(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 1), index(stationIndex, 1) );
}

void stationlistModel::reloadSong(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 2), index(stationIndex, 2) );
}

void stationlistModel::reloadDataSize(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 3), index(stationIndex, 3) );
}

void stationlistModel::reloadBitrate(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 4), index(stationIndex, 4) );
}

void stationlistModel::reloadMetaInterval(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 5), index(stationIndex, 5) );
}

void stationlistModel::reloadServerName(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 6), index(stationIndex, 6) );
}

void stationlistModel::reloadRelayPort(const qlonglong& stationIndex)
{
  emit dataChanged( index(stationIndex, 7), index(stationIndex, 7) );
  actualizeAudioPlayer();
}

QVariant stationlistModel::headerData(int section, Qt::Orientation orientation, int role) const
{
     // code
     if ( orientation EQUAL Qt::Vertical )
     {
       if ( ( section < 0 ) OR ( section >= m_stationlist.size() ) )
         return QVariant();
       else
         return section;
     };

     switch (role)
     {
       case Qt::DisplayRole:
         return columnInfo(section, columnHeaderTitle);
       case Qt::ToolTipRole:
         return columnInfo(section, columnHeaderToolTip);
       case Qt::WhatsThisRole:
         return columnInfo(section, columnHeaderWhatsThis);
     };

     return QVariant(); // if until now there was no valid value found,
                        // we return in ivalid value.

}

QVariant stationlistModel::data(const QModelIndex &index, int role) const
 {
     // variables

     // code
     if ( ( !index.isValid() ) // index has negative row or column count or has no model (okay, WHEN the index has a model, we don't test if it is OUR model...)
          OR ( index.row() >= m_stationlist.size() )
        ) // index is invalid
         return QVariant();

     switch (role)
     {
       case Qt::DisplayRole:
         return columnInfo(index.column(), columnData, index.row());
       case Qt::ToolTipRole:
         return columnInfo(index.column(), columnDataToolTip, index.row());
       case Qt::WhatsThisRole:
         return columnInfo(index.column(), columnDataWhatsThis, index.row());
     }

     return QVariant(); // if index.column wasn't valid, the block befor does nothing.
                        // Also, when a role was used that's not supported, the block does nothing.
                        // In these cases, we return an invalid QVariant.

}

void stationlistModel::addNewStation()
{
  // variables
  const int size_of_list = m_stationlist.size();

  // code
  beginInsertRows(QModelIndex(), size_of_list, size_of_list);
//kDebug() << "stationlist befor" << m_stationlist << endl;
  m_stationlist.append(new radioStation(this, m_mainWidget, QString(), size_of_list));  // generates a new radio station
//kDebug() << "stationlist after" << m_stationlist << endl;
  connectSignalsAndSlots(m_stationlist.last());
  m_stationlist.last()->displaySettingsDialog();
  writeStationListToGeneralSettings();
  endInsertRows();
}

void stationlistModel::deleteStation(const int& index)
{
  if ( (index >= 0) AND (index < m_stationlist.size()) )  // test if the index is valid. (when it is not valid, we may not use QList.at()!!!)
  {
    // variables
    QString filename;
    int i;
    QFile the_file;

    // code
    filename = m_stationlist.at(index)->configFileName();  // save the filename of the config file, we'll need it later to delete the file

    beginRemoveRows(QModelIndex(), index, index);

    delete m_stationlist.at(index);  // delete the radioStation object

    m_stationlist.removeAt(index);  // remove the pointer to the (now deleted) radioStation object from our internal list
    writeStationListToGeneralSettings();  // syncronize the general config file with our internal list
    for (i=index; i<m_stationlist.size(); ++i)  // correct the index for all following rows (which is now changed)
      m_stationlist.at(i)->setIndex(i);

    endRemoveRows();

    // delete the config file of the stream
    the_file.setFileName(filename);
    the_file.remove();

  };
}

inline void stationlistModel::writeStationListToGeneralSettings()
{
  // variables
  int i;
  QStringList helper_stringlist;

  // code
  for (i = 0; i < m_stationlist.size(); ++i)
    helper_stringlist.append(m_stationlist.at(i)->configFileName());
  settings_general::setStreamConfigFiles(helper_stringlist);
  settings_general::self()->writeConfig();
}

void stationlistModel::actualizeAudioPlayer()
{
  kDebug() << "will actualizeAudioPlayer..." << endl;
  // variables
  qint64 helper;
//kDebug() << "function started!" << endl;
kDebug() << "internal_playing is:!" << internal_playing << endl;
  // code
  if ( (internal_playing < 0) OR (internal_playing >= m_stationlist.size()) ) // invalid internal_playing (line number) - think in when a line becomes deleted...
    helper = -1;
  else
    helper = m_stationlist.at(internal_playing)->relayPort();  // get the relay port that SHOULD be used for playback. radioRipper garanties that this value is never < -1.
//kDebug() << "helper (shouldbe) value:" << helper << endl;
  if (internal_actuallyPlaybackedPort != helper) // we have to adjust our settings
  {
kDebug() << "something has changed!" << endl;
kDebug() << "internal value was:" << internal_actuallyPlaybackedPort << endl;
    internal_actuallyPlaybackedPort = helper;  // MUST be called BEFOR m_mediaObject->setCurrentSource() -> see there for the reason.
kDebug() << "internal value is now:" << internal_actuallyPlaybackedPort << endl;
kDebug() << "I'll call stop().." << endl;
      m_mediaObject->stop();
kDebug() << "done!" << endl;
    if (internal_actuallyPlaybackedPort > (-1))
    {
      m_mediaObject->setCurrentSource(Phonon::MediaSource(KUrl(   QString("http://127.0.0.1:%1").arg(internal_actuallyPlaybackedPort)   )));
kDebug() << "source set to:" << m_mediaObject->currentSource().url() << endl;
      // we will not call m_mediaObject->play(); here because after setting the source, m_mediaObject needs some time to get ready.
      // When it is ready, the slot process_new_mediaObjectState(Phonon::State newstate) - which is connected (in the constructor) with the signal stateChanged() of m_mediaObject will - take care of starting it, but only in case that internal_actuallyPlaybackedPort is > 0. (Because it is the same state when m_mediaObject becomes ready and when it was stopped, we need this to distingish if we should really start.)
    };
  }
}

void stationlistModel::process_new_mediaObjectState ( Phonon::State newstate)
{
kDebug() << "new phonon state" << newstate << endl;
kDebug() << "internal_actuallyPlaybackedPort: " << internal_actuallyPlaybackedPort << endl;
  if ( (newstate EQUAL Phonon::StoppedState) AND (internal_actuallyPlaybackedPort != -1) )  // when 
{ kDebug() << "so I will call play()" << endl;
    m_mediaObject->play();
};
}

void stationlistModel::record(const int& index)
{
  if ( (index >= 0) AND (index < m_stationlist.size()) )  // test if the index is valid. (when it is not valid, we may not use QList.at()!!!)
    m_stationlist.at(index)->startStreamripper();
}

void stationlistModel::stopRecording(const int& index)
{
  if ( (index >= 0) AND (index < m_stationlist.size()) )  // test if the index is valid. (when it is not valid, we may not use QList.at()!!!)
    m_stationlist.at(index)->shutDown();
}

void stationlistModel::displayConfigDialog(const int& index)
{
  if ( (index >= 0) AND (index < m_stationlist.size()) )  // test if the index is valid. (when it is not valid, we may not use QList.at()!!!)
    m_stationlist.at(index)->displaySettingsDialog();
}

int stationlistModel::playing() const
{
//kDebug() << "playing: my internal_playing: " << internal_playing << endl;
  return internal_playing;
}

void stationlistModel::setPlaying(int index)
{
  if ((index < -1) OR (index >= m_stationlist.size()) ) index = -1;  // value < -1 are treated like -1 (which means stopping the playback). The same is done when the given index is bigger than the last row (= invalid index).
  if ( internal_playing != index )  // we have to do something because the index has changed...
  {
    internal_playing = index;
    actualizeAudioPlayer();
  };
}

void stationlistModel::setPlayingModelIndex(const QModelIndex & index)
{
  setPlaying(index.row());
}

