// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "stationlistwidget.h"
#include "settings_general.h"
#include "stationlistmodel.h"
#include <QTimer>
#include <QHeaderView>
#include <QContextMenuEvent>
#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>

#include "m_header.h"

// TODO save which columns are visible/unvisible
// TODO more columns

stationlistWidget::stationlistWidget(QWidget *parent, QPointer<QWidget> mainWindow) : QTableView(parent)
{
  m_mainWindow = mainWindow;
  QTimer::singleShot( 0, this, SLOT(delayedInitialization()) );

  m_horizontalHeader = new CustomizableHeaderView(Qt::Horizontal);
  setHorizontalHeader(m_horizontalHeader);
  m_horizontalHeader->setContextMenuTitle(i18nc("TODO", "Columns")); // TODO teste den Einsatz von CustomizableHeaderView als vertikalen Header!!!
  connect(m_horizontalHeader,
          SIGNAL(   sectionAboutToBeHidden(int, Qt::Orientation)   ),
          this,
          SLOT(   saveSectionSize(int, Qt::Orientation)   )
         );
}

void stationlistWidget::delayedInitialization()
{
  setSelectionBehavior(QAbstractItemView::SelectRows);  // don't change this. Much code relys on the assumption that only hole rows can be selected.
  setShowGrid(false);
  m_stationlistModel = new stationlistModel(this, m_mainWindow);
  setModel(m_stationlistModel);
  verticalHeader()->setVisible(true);  // TODO why hasn't this any effect???
  
  // variables
  int i;

  // code
  for (i=0; i < m_stationlistModel->columnCount(); i++)
  {
    setColumnHidden(   i,   NOT m_stationlistModel->columnInfo(i, stationlistModel::columnVisibility).toBool()   );
    setColumnWidth(   i,   m_stationlistModel->columnInfo(i, stationlistModel::columnWidth).toLongLong()   );
  };
}

stationlistWidget::~stationlistWidget()
{
  // variables
  int i;

  // code
  for (i=0; i < m_stationlistModel->columnCount(); i++)
  {
    if (NOT isColumnHidden(i))
      m_stationlistModel->columnInfo(i, stationlistModel::setColumnWidth, columnWidth(i));
    m_stationlistModel->columnInfo(i, stationlistModel::setColumnVisibility, NOT isColumnHidden(i));
  };
}

void stationlistWidget::saveSectionSize(int index, Qt::Orientation orientation)
{
  if (orientation EQUAL Qt::Horizontal)
    m_stationlistModel->columnInfo(index, stationlistModel::setColumnWidth, columnWidth(index));
}

void stationlistWidget::record()
{
  // variables
  int i;
  QModelIndexList m_list = selectionModel()->selectedRows();  // initialized with a list of the selected rows (for column 0)

  // code
  for (i = 0; i < m_list.size(); ++i)
    m_stationlistModel->record(m_list.at(i).row());
}

void stationlistWidget::stopRecord()
{
  // variables
  int i;
  QModelIndexList m_list = selectionModel()->selectedRows();  // initialized with a list of the selected rows (for column 0)

  // code
  for (i = 0; i < m_list.size(); ++i)
    m_stationlistModel->stopRecording(m_list.at(i).row());
}

/*void stationlistWidget::currentChanged ( const QModelIndex & current, const QModelIndex & previous )
{
  QTableView::currentChanged(current, previous);
  m_stationlistModel->setPlaying(current.row());
}*/

void stationlistWidget::selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )
{
  // call the original function from the base class
  QTableView::selectionChanged(selected, deselected);

  // variables
  QModelIndexList m_list = selectionModel()->selectedRows();  // initialized with a list of the selected rows (for column 0)

  // code
  switch (m_list.size())
  {
    case 0:  {
               emit noneSelected(true);
               m_stationlistModel->setPlaying(-1);
               break;
             }
    case 1:  {
               emit oneSelected(true);
               m_stationlistModel->setPlayingModelIndex(m_list.at(0));
               break;
             }
    default: {
               emit multipleSelected(true);
               m_stationlistModel->setPlaying(-1);
               break;
             }
  };
}

void stationlistWidget::displayStreamSettings()
{
  // variables
  QModelIndexList m_list = selectionModel()->selectedRows();  // initialized with a list of the selected rows (for column 0)

  // code
  if (m_list.size() == 1)
    m_stationlistModel->displayConfigDialog(m_list.at(0).row());
}

void stationlistWidget::deleteStream()
{
  // variables
  QModelIndexList m_list;

  // code
  m_list = selectionModel()->selectedRows();  // initialized with a list of the selected rows (for column 0)
  while (m_list.size() > 0)
  {
    m_stationlistModel->deleteStation(m_list.at(0).row());
    m_list = selectionModel()->selectedRows();
  };
}

void stationlistWidget::addNewStation()
{
  m_stationlistModel->addNewStation();
}

void stationlistWidget::contextMenuEvent(QContextMenuEvent * e)
{
  contextMenu.exec(e->globalPos());
}
