// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#ifndef STATIONLISTWIDGET_H
#define STATIONLISTWIDGET_H
 
#include <QTableView>
#include <QPointer>
#include <KMenu>
#include "stationlistmodel.h"
#include "customizableheaderview.h"

/** This class inherits \e QTableView. It provides a \e QTableView
*   using by default a \e stationlistModel object as model.
*   Furthermore it provides nicer default values than \e QTableView
*   for displaying the data and handels automaticly the playback. 
*   Attention: NEVER change the model of this view!   // TODO make shure that setModel() becomes private!!!
*   Attention: Make shure that you have only ONE object of this class at the same time.
*   Attention: This class uses delayed initialatition - will say, the object works not directly after the construction,
*   put posts an event to the event loop which is processed than. This way the startup is faster. Make shure to
*   not call any function apart (from the constructor) befor the event loop is running. */
class stationlistWidget : public QTableView
 {
     Q_OBJECT

 public:
     // contructor(s) and destructor(s)
     /** The constructor. */
     stationlistWidget(QWidget *parent = 0, QPointer<QWidget> mainWindow = 0);  // using QWidget* instead of QPointer<QWidget> to make type conversion to QObject* possible
     virtual ~stationlistWidget();
     KMenu contextMenu;
 signals:
     /** This signal is emited when the selection changes and now none of the rows is selected.
     *   @param value is always TRUE. */
     void noneSelected(bool value);
     /** This signal is emited when the selection changes and now exactly one of the rows is selected.
     *   @param value is always TRUE. */
     void oneSelected(bool value);
     /** This signal is emited when the selection changes and now more than one of the rows are selected.
     *   @param value is always TRUE. */
     void multipleSelected(bool value);

 public slots:
     void record();
     void stopRecord();
     void displayStreamSettings();
     void deleteStream();
     void addNewStation();
     //QPointer<QMenu> contextmenu_visible_columns();

protected:
     /** Reimplemented from class QAbstractScrollArea. */
     virtual void contextMenuEvent ( QContextMenuEvent * e );

 protected slots:
//     virtual void currentChanged ( const QModelIndex & current, const QModelIndex & previous );
     virtual void selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected );

 private:
     QPointer<stationlistModel> m_stationlistModel;
     QPointer<QWidget> m_mainWindow;
     QPointer<CustomizableHeaderView> m_horizontalHeader;

 private slots:
     /** This function sets up the data model. The constructor of the class will call this using the event loop - AFTER showing the mainwindow.
     *   This seems faster to the user. */
     void delayedInitialization();
     void saveSectionSize(int index, Qt::Orientation orientation);
 }; 

#endif
